#Klasa Razlomak
class Razlomak:
    '''Klasa Razlomak omogućuje deklariranje razlomaka i provođenje četiri
    osnovne operacije s pripadnim operatorima:
        + za zbrajanje (spec. metoda __add__()),
        - za oduzimanje (spec.metoda __sub__()),
        * za množenje (spec. metoda __mul__()),
        / za dijeljenje (spec. metoda __truediv__()),
    operacija:
        += (spec. metoda __iadd__())
        *= (spec. metoda __imul__())
        ** (spec. metoda __pow__()),
    te relacijske operacije:
        < (spec. metoda __lt__())
        > (spec. metoda __gt__())
        == (spec. metoda __eq__())
        != (spec. metoda __ne__())
     Klasa podržava i funkciju
        float() (spec. metoda __float__())
     Klasa ima svoj konstruktor __init__ koja elimira eventualni zajendički
     djelitelj unesen prilikom konstrukcije (skraćuje razlomak).
     Spec. metoda __str__() ispisuje razlomak u obliku:
         brojnik/nazivnik.'''    

    def __init__(self, b = 0, n = 1):						#1
        self.b = b
        self.n = n
        self.krati()
        return

    def krati(self):									#6
        b = self.b
        n = self.n        
        while b % n != 0:
            b, n = n, b % n
        t = n
        self.b //= t
        self.n //= t
        return

    def __str__(self):									#7
        if self.n == 1:
            return '{}'.format(self.b)
        else:
            return '{0} / {1}'.format(self.b, self.n)

    def __repr__(self):									#8
        return self.__str__()


    def __add__(self, r):								#9
        b = self.b * r.n + r.b * self.n
        n = self.n * r.n
        return Razlomak(b, n)

    def __sub__(self, r):								#10
        b = self.b * r.n - r.b * self.n
        n = self.n * r.n        
        return Razlomak(b, n)

    def __mul__(self, r):								#11
        b = self.b * r.b
        n = self.n * r.n        
        return Razlomak(b, n)

    def __truediv__(self, r):								#12
        b = self.b * r.n
        n = self.n * r.b        
        return Razlomak(b, n)

    def __iadd__(self, r):								#13
        return self.__add__(r)

    def __imul__(self, r):								#14
        return self.__mull__(r)

    def __isub__(self, r):								#15
        return self.__sub__(r)

    def __pow__(self, potencija):							#16
        if potencija < 0:
            b, n = self.n, self.b
            potencija = abs(potencija)
        else:
            b, n = self.b, self.n
        return Razlomak(b ** potencija, n ** potencija)

    def __lt__(self, r):								#17
        return self.b * r.n < self.n * r.b

    def __eq__(self, r):								#18
        return self.b * r.n == self.n * r.b

    def __gt__(self, r):								#19
        return self.b * r.n > self.n * r.b

    def __ﬂoat__(self):
        return self.b/self.n
        

def main():
    a = Razlomak(3,7)								     #20
    b = Razlomak(5,11) 								     #21
    print('a = {}'.format(a))
    print('b = {}'.format(b))
    print('zbrajanje')
    print('{0} + {1} = {2}'.format(a, b , a + b))
    print('oduzimanje')
    print('{0} - {1} = {2}'.format(a, b , a - b))
    print('množenje')
    print('{0} * {1} = {2}'.format(a, b , a * b))
    print('dijeljenje')
    print('({0}) / ({1}) = {2}'.format(a, b , a / b))
    print('potenciranje')
    print('({0}) ** 2 = {1}'.format(a, a ** 2))
    print('({0}) ** -1 = {1}'.format(a, a ** -1)) 
    print('uspoređivanje')
    print('{0} > {1} -> {2}'.format(a, b, a > b))
    print('{0} < {1} -> {2}'.format(a, b, a < b))
    print('{0} == {1} -> {2}'.format(a, b, a == b))
    print('{0} != {1} -> {2}'.format(a, b, a != b))
    print('pretvorba racionalnog broja u broj s pomičnom točkom')
    print('float({0}) = {1}'.format(a, float(a)))

if __name__ == '__main__':
    main()
